@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -27,6 +27,10 @@
 
 #include <glib.h>
 
+G_BEGIN_DECLS
+
 struct tm *lm_utils_get_localtime (const gchar *stamp);
 
+G_END_DECLS
+
 #endif /* __LM_UTIL__ */
